'use strict';

// YOU KNOW WHAT TO DO //

/**
 * each: Designed to loop over a collection, Array or Object, and applies the 
 * action Function to each value in the collection.
 * 
 * @param {Array or Object} collection: The collection over which to iterate.
 * @param {Function} action: The Function to be applied to each value in the 
 * collection
 */
function each(collection, action) {
    if(Array.isArray(collection)) {
        for(var i = 0; i < collection.length; i++) {
            action(collection[i], i, collection);
        }
    } else {
        for (var key in collection) {
            action(collection[key], key, collection);
        }
    }
}
module.exports.each = each;


/**
 * identity: Designed to return the same value given to it as an argument.
 * 
 * @param {any} arg: The value to be returned.
 * 
 * Usage:
 *      let val = 5;
 *      console.log(_.identity(val)) // -> 5
 */
 _.identity = function(arg) {
    return arg;
}
module.exports.identity = _.identity;


/**
 * typeOf: Designed to return the datatype of the provided argument as a string.
 * @param {*} arg: The value to return the datatype of.
 * 
 * 
 */
_.typeOf = function(arg) {
    switch (typeof arg) {
        case "number":
            return "number";
            break;
        case "string":
            return "string";
            break;
        case "boolean":
            return "boolean";
            break;
        case "function":
            return "function";
            break;
        case "object":
            if (Array.isArray(arg)) {
                return "array";
            } else if (arg == null) {
                return "null";
            }
            return "object";
            break;
        default:
            return "undefined";
            break;
    }
}
module.exports.typeOf = _.typeOf;


/**
 * first: Returns the first (num) values of the array (arr), or the entire array if num is too large. Returns an empty array when num < 1 or arr isnt an Array.
 * @param {Array} arr: The array to take the values from.
 * @param {Number} num: The number of values to take from arr (starting from 1).
 */
_.first = function(arr, num) {
    let retArr = [];
    if (!Array.isArray(arr) || num < 1) return [];
    if (typeof num !== "number" || num === 1) return arr[0];
    for (let i = 0; i < Math.min(num, arr.length); i++) {
        retArr.push(arr[i]);
    }
    return retArr;
}
module.exports.first = _.first;


/**
 * last: Returns the last (num) values of the array (arr), or the entire array if num is too large. Returns an empty array when arr isnt an Array. Returns the last value of arr when num isnt a Number.
 * @param {Array} arr: The array to take the values from.
 * @param {Number} num: The number of values to take from arr (starting from 1).
 */
_.last = function(arr, num) {
    let retArr = [];
    if (!Array.isArray(arr)) return [];
    if (typeof num !== "number" || num === 1) return arr[arr.length-1];
    for (let i = Math.max(arr.length-num,0); i < arr.length; i++) {
        retArr.push(arr[i]);
    }
    return retArr;
}
module.exports.last = _.last;

/**
 * indexOf: Returns the index of the first occurence of (val) within (arr). Returns -1 if (val) does not exist in (arr).
 * @param {Array} arr: The array to look through.
 * @param {*} val: The value to look for.
 */
_.indexOf = function(arr, val) {
    for (let i = 0; i < arr.length; i++) {
        if (arr[i] == val) return i;
    }
    return -1;
}
module.exports.indexOf = _.indexOf;


/**
 * contains: Returns a bool for whether or not (arr) contains (val).
 * @param {Array} arr: The array to look through.
 * @param {*} val: The value to look for.
 */
_.contains = function(arr, val) {
    let contains = false;

    for (let i = 0; i < arr.length; i++) {

        contains = (arr[i] === val ? true : contains);

    }
    return contains;
}
module.exports.contains = _.contains;


/**
 * unique: Returns an array containing all unique values within (arr).
 * @param {Array} arr 
 */
_.unique = function(arr) {
    let retArr = [];
    for (let i = 0; i < arr.length; i++) {
        if (_.indexOf(retArr, arr[i]) == -1) retArr.push(arr[i]);
    }
    return retArr;
}
module.exports.unique = _.unique;

/**
 * filter: Returns an array containing all values of (arr) for which func(arr[i]) returns true.
 * @param {Array} arr 
 * @param {Function} func 
 */
_.filter = function(arr, func) {
    let retArr = [];
    function f(v, i, r) {
        if (func(v, i, r) == true) retArr.push(v);
    }
    _.each(arr, f);
    return retArr;
}
module.exports.filter = _.filter;

/**
 * reject: Returns an array containing all values of (arr) for which func(arr[i]) returns false.
 * @param {Array} arr 
 * @param {Function} func 
 */
_.reject = function(arr, func) {
    let retArr = [];
    function f(v, i, r) {
        if (func(v, i, r) == false) retArr.push(v);
    }
    _.each(arr, f);
    return retArr;
}
module.exports.reject = _.reject;

/**
 * partition: Returns an array containing two arrays.
 * Array[0] is an array containing all values of (arr) for which func(arr[i]) returns true.
 * Array[1] is an array containing all values of (arr) for which func(arr[i]) returns false.
 * @param {Array} arr 
 * @param {Function} func 
 */
_.partition = function(arr, func) {
    let bigArr = [];
    function part(arr, func, bool) {
        let retArr = [];
        function f(v, i, r) {
            if (func(v, i, r) == bool) retArr.push(v);
        }
         _.each(arr, f);
        return retArr;
    }
    bigArr.push(part(arr, func, true));
    bigArr.push(part(arr, func, false));
    return bigArr;
}
module.exports.partition = _.partition;

/**
 * map: Returns an array containing the results of passing each value of (coll) through (func).
 * @param {Array|Object} coll 
 * @param {Function} func 
 */
_.map = function(coll, func) {
    let retArr = [];
    if (Array.isArray(coll)) {
        for (let i = 0; i < coll.length; i++) {
            retArr.push(func(coll[i], i, coll));
        }
    } else {
        for (let key in coll) {
            retArr.push(func(coll[key], key, coll));
        }
    }
    return retArr;
}
module.exports.map = _.map;


/**
 * pluck: Returns an array of the values of key (prop) from each object in (arr).
 * @param {Object} arr 
 * @param {String} prop 
 */
_.pluck = function(arr, prop) {
    let retArr = [];
    function f(v,i,c) {
        for (let key in v) {
            if (key == prop) retArr.push(v[key]);
        }
    }
    _.map(arr, f);
    return retArr;
}
module.exports.pluck = _.pluck;


/**
 * every: Returns true if func(coll[i]) returns true for every value in (coll), or if every 
  * @param {*} coll 
 * @param {*} func 
 * @returns 
 */
_.every = function(coll, func) {
    if (Array.isArray(coll)) {
        for (let i = 0; i < coll.length; i++) {
            if (func == undefined) {
                console.log(coll);
                if (!coll[i]) return false;
            } else if (func(coll[i], i, coll) == false) return false;
        }
        return true;
    } else {
        for (let key in coll) {
            if (func == undefined) {
                if (!coll[i]) return false;
            } else if (func(coll[key], key, coll) == false) return false;
        } 
        return true;
    }
}