'use strict';

// YOU KNOW WHAT TO DO //

/**
 * each: Designed to loop over a collection, Array or Object, and applies the 
 * action Function to each value in the collection.
 * 
 * @param {Array or Object} collection: The collection over which to iterate.
 * @param {Function} action: The Function to be applied to each value in the 
 * collection
 */
function each(collection, action) {
    if(Array.isArray(collection)) {
        for(var i = 0; i < collection.length; i++) {
            action(collection[i], i, collection);
        }
    } else {
        for (var key in collection) {
            action(collection[key], key, collection);
        }
    }
}
module.exports.each = each;


/**
 * identity: Designed to return the same value given to it as an argument.
 * 
 * @param {any} arg: The value to be returned.
 * @returns {any}
 * 
 * Usage:
 *      let val = 5;
 *      console.log(_.identity(val)) // -> 5
 */
 _.identity = function(arg) {
    return arg;
}
module.exports.identity = _.identity;


/**
 * typeOf: Designed to return the datatype of the provided argument as a string.
 * @param {*} arg: The value to return the datatype of.
 * @returns {string} : The datatype of the argument as a string
 */
_.typeOf = function(arg) {
    switch (typeof arg) {
        case "number":
            return "number";
            break;
        case "string":
            return "string";
            break;
        case "boolean":
            return "boolean";
            break;
        case "function":
            return "function";
            break;
        case "object":
            if (Array.isArray(arg)) {
                return "array";
            } else if (arg == null) {
                return "null";
            }
            return "object";
            break;
        default:
            return "undefined";
            break;
    }
}
module.exports.typeOf = _.typeOf;


/**
 * first: Returns the first (num) values of the array (arr), or the entire array if num is too large. Returns an empty array when num < 1 or arr isnt an Array.
 * @param {Array} arr 
 * @param {Number} num 
 * @returns {Array}
 */
_.first = function(arr, num) {
    let retArr = [];
    if (!Array.isArray(arr) || num < 1) return [];
    if (typeof num !== "number" || num === 1) return arr[0];
    for (let i = 0; i < Math.min(num, arr.length); i++) {
        retArr.push(arr[i]);
    }
    return retArr;
}
module.exports.first = _.first;


/**
 * first: Returns the last (num) values of the array (arr), or the entire array if num is too large. Returns an empty array when num < 1 or arr isnt an Array.
 * @param {Array} arr 
 * @param {Number} num 
 * @returns {Array}
 */
_.last = function(arr, num) {
    let retArr = [];
    if (!Array.isArray(arr)) return [];
    if (typeof num !== "number" || num === 1) return arr[arr.length-1];
    for (let i = Math.max(arr.length-num,0); i < arr.length; i++) {
        retArr.push(arr[i]);
    }
    return retArr;
}